#include <SPI.h>
#include <Ethernet.h>
#include <EEPROM.h>
#include "EEPROMAnything.h"
#include <avr/wdt.h>
#include <string.h>

#define EN 42

long readTemp() { 
  // Read temperature sensor against 1.1V reference
  #if defined(__AVR_ATmega32U4__)
    ADMUX = _BV(REFS1) | _BV(REFS0) | _BV(MUX2) | _BV(MUX1) | _BV(MUX0);
    ADCSRB = _BV(MUX5); // the MUX5 bit is in the ADCSRB register
  #elif defined (__AVR_ATtiny24__) || defined(__AVR_ATtiny44__) || defined(__AVR_ATtiny84__)
    ADMUX = _BV(REFS1) | _BV(MUX5) | _BV(MUX1);
  #elif defined (__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__)
    ADMUX = _BV(REFS1) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1) | _BV(MUX0);
  #else
    ADMUX = _BV(REFS1) | _BV(REFS0) | _BV(MUX3);
  #endif

  delay(2); // Wait for ADMUX setting to settle
  ADCSRA |= _BV(ADSC); // Start conversion
  while (bit_is_set(ADCSRA,ADSC)); // measuring

  uint8_t low = ADCL; // must read ADCL first - it then locks ADCH
  uint8_t high = ADCH; // unlocks both
  long result = (high << 8) | low; // combine the two

  return result;
}

float normalizeTemperature(long rawData) { 
  // replace these constants with your 2 data points
  // these are sample values that will get you in the ballpark (in degrees C)
  float temp1 = 0;
  long data1 = 274;
  float temp2 = 25.0;
  long data2 = 304;
 
  // calculate the scale factor
  float scaleFactor = (temp2 - temp1) / (data2 - data1);

  // now calculate the temperature
  float temp = scaleFactor * (rawData - data1) + temp1;

  return temp;
}

long readVcc() {
  // Read 1.1V reference against AVcc
  // set the reference to Vcc and the measurement to the internal 1.1V reference
  #if defined(__AVR_ATmega32U4__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
    ADMUX = _BV(REFS0) | _BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #elif defined (__AVR_ATtiny24__) || defined(__AVR_ATtiny44__) || defined(__AVR_ATtiny84__)
     ADMUX = _BV(MUX5) | _BV(MUX0) ;
  #else
    ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  #endif  
 
  delay(2); // Wait for Vref to settle
  ADCSRA |= _BV(ADSC); // Start conversion
  while (bit_is_set(ADCSRA,ADSC)); // measuring
 
  uint8_t low  = ADCL; // must read ADCL first - it then locks ADCH  
  uint8_t high = ADCH; // unlocks both
 
  long result = (high<<8) | low;
 
  result = 1125300L / result; // Calculate Vcc (in mV); 1125300 = 1.1*1023*1000
  return result-50; // Vcc in millivolts
}

struct settings_p
{
  bool Fst;
  long hcol, ccol;
  int w;
  unsigned char r, g, b, save_sucsess, Sst;
} settings;


int save_settings(bool en)
{
  settings.hcol=htmp;
  settings.ccol=ctmp;
  settings.Fst=Fst;
  settings.Sst=Sst;
  //settings.r=r;
  //settings.g=g;
  //settings.b=b;
  //settings.w=w;
  if(en==1) {
   settings.save_sucsess=EN;}
  if(en==0) {
   settings.save_sucsess=25;}
  EEPROM_writeAnything(21, settings);
}

int restore_settings()
{
  EEPROM_readAnything(21, settings);
  if(settings.save_sucsess==EN) {    
   htmp=settings.hcol;
   ctmp=settings.ccol;
   Fst=settings.Fst;
   Sst=settings.Sst;
   //r=settings.r;
   //g=settings.g;
  // b=settings.b;
  // w=settings.w;
   load_sucsess=1;
   return 0;
  }
  else
  {
    #if defined(SERIAL)
    //Serial.println("Error can't resore sittings, using default.");
    #endif
    load_sucsess=0;
    return 1;
  }
}

void autosave() {
  uptime = millis();
  if(uptime - oldtime > 1200000) {
    save_settings(1);
  }
  oldtime = uptime;
  if(i>20) {
    save_settings(1);
    i=0;
  }
  else {
   i++; 
  }
  status_update=true;
}

void setup() {
  wdt_disable();
  
  restore_settings();
  
  RGBwr();
  digitalWrite(8, Fst);
  
  // start the Ethernet connection and the server:
  //Ethernet.begin(mac);
  #if defined(SERIAL)
  //Serial.print("server is at ");
  //Serial.println(Ethernet.localIP());
  #endif
  save_settings(1);
  delay(200);
  attachInterrupt(0, hot, RISING);
  attachInterrupt(1, cold, RISING);
}

void loop() {
  
  
  
}
